@@ -40,28 +40,48 @@ Taxmap <- R6::R6Class(
     data = list(),
     funcs = list(),
 
+    # -------------------------------------------------------------------------
+    # Constructor
     initialize = function(..., .list = NULL, data = list(), funcs = list()) {
 
       # Call `taxonomy` constructor
       super$initialize(..., .list = .list)
 
       # Make sure `data` is in the right format and add to object
-      self$data <- validate_taxmap_data(data, self$input_ids)
+      self$data <- init_taxmap_data(self, data, self$input_ids)
       check_taxmap_data(self)
 
       # Make sure `funcs` is in the right format and add to object
       self$funcs <- validate_taxmap_funcs(funcs)
     },
 
+    # -------------------------------------------------------------------------
     print = function(indent = "", max_rows = 3, max_items = 6,
                      max_width = getOption("width") - 10) {
 
-      # Call `taxonomy` print method
-      taxonomy_output <- paste0(
-        paste0(capture.output(super$print(indent = indent)), collapse = "\n"),
-        "\n"
-      )
-      cat(gsub(taxonomy_output, pattern = "Taxonomy", replacement = "Taxmap"))
+      # Print taxonomy
+      cat(paste0(indent, "<Taxmap>\n"))
+      taxon_names <- vapply(self$taxa, function(x) x$name$name, character(1))
+      taxon_ids <- names(self$taxa)
+      if (length(self$taxa) > 0) {
+        limited_print(paste(tid_font(taxon_ids), taxon_names,
+                            sep = punc_font(". ")),
+                      sep = punc_font(", "),
+                      mid = punc_font(" ... "),
+                      trunc_char = punc_font("[truncated]"),
+                      prefix = paste0(indent, "  ",
+                                      length(self$taxa), " taxa:"),
+                      type = "cat")
+        limited_print(private$make_graph(),
+                      sep = punc_font(", "),
+                      mid = punc_font(" ... "),
+                      trunc_char = punc_font("[truncated]"),
+                      prefix = paste0(indent, "  ",
+                                      nrow(self$edge_list), " edges:"),
+                      type = "cat")
+      } else {
+        cat("  No taxa\n  No edges\n")
+      }
 
       # Get item names
       if (is.null(names(self$data))) {
@@ -69,14 +89,14 @@ Taxmap <- R6::R6Class(
       } else {
         data_names <- names(self$data)
         data_names[data_names == ""] <- paste0("[[", which(data_names == ""),
-                                                "]]")
+                                               "]]")
       }
 
       # Print a subset of each item, up to a max number, then just print names
       cat(paste0("  ", length(self$data), " data sets:\n"))
       if (length(self$data) > 0) {
         for (i in 1:min(c(max_items, length(self$data)))) {
-          print_item(self$data[[i]],
+          print_item(self, self$data[[i]],
                      name = data_names[i], max_rows = max_rows,
                      max_width = max_width, prefix = "    ")
         }
@@ -90,11 +110,19 @@ Taxmap <- R6::R6Class(
 
       # Print the names of functions
       cat(paste0("  ", length(self$funcs), " functions:\n"))
-      limited_print(names(self$funcs), type = "cat")
+      limited_print(prefix = "   ", names(self$funcs), type = "cat")
 
       invisible(self)
     },
 
+    # -------------------------------------------------------------------------
+    # Check that a set of IDs are valid taxon IDs
+    is_taxon_id = function(ids) {
+      valid_ids <- c(unlist(self$edge_list), names(self$taxa), NA)
+      ids %in% valid_ids
+    },
+
+    # -------------------------------------------------------------------------
     # Returns the names of things to be accessible using non-standard evaluation
     all_names = function(tables = TRUE, funcs = TRUE, others = TRUE,
                          builtin_funcs = TRUE, warn = FALSE) {
@@ -150,13 +178,15 @@ Taxmap <- R6::R6Class(
     },
 
 
+    # -------------------------------------------------------------------------
+    # Get data indexes or other values associated with taxa
     obs = function(data, value = NULL, subset = NULL, recursive = TRUE,
                    simplify = FALSE) {
       # non-standard argument evaluation
       data_used <- eval(substitute(self$data_used(subset)))
       subset <- lazyeval::lazy_eval(lazyeval::lazy(subset), data = data_used)
       subset <- private$parse_nse_taxon_subset(subset)
-      obs_taxon_ids <- private$get_data_taxon_ids(data, require = TRUE)
+      obs_taxon_ids <- self$get_data_taxon_ids(data, require = TRUE)
 
       # Get observations of taxa
       if (is.logical(recursive) && recursive == FALSE) {
@@ -186,19 +216,22 @@ Taxmap <- R6::R6Class(
         if (is.null(names(possible_values))) {
           output <- lapply(output, function(i) possible_values[i])
         } else {
-          output <- lapply(output, function(i) possible_values[private$get_data_taxon_ids(data)[i]])
+          output <- lapply(output, function(i) possible_values[self$get_data_taxon_ids(data)[i]])
         }
       }
 
       # Reduce dimensionality
       if (simplify) {
-        output <- unique(unname(unlist(output)))
+        output <- simplify(output)
       }
 
       return(output)
     },
 
 
+    # -------------------------------------------------------------------------
+    # Apply a function to data for the observations for each taxon.
+    # This is similar to using obs() with lapply() or sapply().
     obs_apply = function(data, func, simplify = FALSE, value = NULL,
                          subset = NULL, recursive = TRUE, ...) {
       my_obs <- self$obs(data, simplify = FALSE, value = value,
@@ -206,20 +239,30 @@ Taxmap <- R6::R6Class(
                          recursive = recursive)
       output <- lapply(my_obs, func, ...)
       if (simplify) {
-        output <- unlist(output)
+        output <- simplify(output)
       }
       return(output)
     },
 
-
-    filter_obs = function(target, ..., drop_taxa = FALSE) {
+    # -------------------------------------------------------------------------
+    # Filter data in a taxmap() object (in obj$data) with a set of conditions.
+    filter_obs = function(target, ..., drop_taxa = FALSE, drop_obs = TRUE,
+                          subtaxa = FALSE, supertaxa = TRUE,
+                          reassign_obs = FALSE) {
       # Check that the target data exists
-      private$check_dataset_name(target)
+      private$is_datset(target)
 
       # non-standard argument evaluation
       selection <- lazyeval::lazy_eval(lazyeval::lazy_dots(...),
                                        data = self$data_used(...))
 
+      # Parse drop_obs option
+      drop_obs <- parse_possibly_named_logical(
+        drop_obs,
+        self$data,
+        formals(self$filter_obs)$drop_obs
+      )
+
       # If no selection is supplied, match all rows
       if (length(selection) == 0) {
         selection <- list(seq_len(nrow(self$data[[target]])))
@@ -239,36 +282,39 @@ Taxmap <- R6::R6Class(
       selection[is_logical] <- lapply(selection[is_logical], which)
 
       # combine filters
-      intersect_with_dups <-function(a, b) {
+      intersect_with_dups <- function(a, b) {
         rep(sort(intersect(a, b)), pmin(table(a[a %in% b]), table(b[b %in% a])))
       }
       selection <- Reduce(intersect_with_dups, selection)
 
       # Remove observations
-      data_taxon_ids <- private$get_data_taxon_ids(target)
+      data_taxon_ids <- self$get_data_taxon_ids(target, require = drop_taxa)
       private$remove_obs(dataset = target, indexes = selection)
 
       # Remove unobserved taxa
       if (drop_taxa & ! is.null(data_taxon_ids)) {
-        unobserved_taxa <- self$supertaxa(unique(data_taxon_ids[-selection]),
-                                          na = FALSE, recursive = TRUE,
-                                          simplify = TRUE, include_input = TRUE,
-                                          value = "taxon_indexes")
-        taxa_to_remove <- 1:nrow(self$edge_list) %in%
-          unobserved_taxa & vapply(self$obs(target), length, numeric(1)) == 0
-        self$taxa <- self$taxa[self$taxon_ids()[! taxa_to_remove]]
-        self$edge_list <- self$edge_list[! taxa_to_remove, , drop = FALSE]
-        self$edge_list[! self$edge_list$from %in% self$taxon_ids(), "from"] <-
-          as.character(NA)
+
+        # dont remove taxa that appear in other data sets if they are not also filtered
+        sets_to_keep_ids_from <- names(drop_obs[! drop_obs])
+        other_ids_to_keep <- unique(unlist(lapply(sets_to_keep_ids_from,
+                                                  self$get_data_taxon_ids)))
+        taxon_ids_to_keep <- unique(c(data_taxon_ids[selection], other_ids_to_keep))
+
+        # Remove taxa that are not in the filtered data set
+        self$filter_taxa(taxon_ids_to_keep, drop_obs = drop_obs,
+                         subtaxa = subtaxa, supertaxa = supertaxa,
+                         reassign_obs = reassign_obs)
       }
 
       return(self)
     },
 
 
+    # -------------------------------------------------------------------------
+    # Subsets columns in a data set
     select_obs = function(target, ...) {
       # Check that the target data exists
-      private$check_dataset_name(target)
+      private$is_datset(target)
 
       # Check that the target is a table
       if (! is.data.frame(self$data[[target]])) {
@@ -282,30 +328,69 @@ Taxmap <- R6::R6Class(
     },
 
 
+    # -------------------------------------------------------------------------
+    # Add columns to tables in obj$data
     mutate_obs = function(target, ...) {
-      # Check that the target data exists
-      private$check_dataset_name(target)
-
-      # Check that the target is a table
-      if (! is.data.frame(self$data[[target]])) {
-        stop(paste0('The dataset "', target, '" is not a table, so columns cannot be selected.'))
-      }
 
+      # Get data used in expressions to add
       data_used <- self$data_used(...)
       unevaluated <- lazyeval::lazy_dots(...)
-      for (index in seq_along(unevaluated)) {
-        new_col <- lazyeval::lazy_eval(unevaluated[index], data = data_used)
-        # Allow this col to be used in evaluating the next cols
-        data_used <- c(data_used, new_col)
-        self$data[[target]][[names(new_col)]] <- new_col[[1]]
+
+      # add columns
+      if (private$is_datset(target, require = FALSE)) {
+        # Check that the target is a table
+        if (! is.data.frame(self$data[[target]])) {
+          stop(paste0('The dataset "', target, '" is not a table, so columns cannot be added'))
+        } else {
+          for (index in seq_along(unevaluated)) {
+            new_col <- lazyeval::lazy_eval(unevaluated[index], data = data_used)
+            data_used <- c(data_used, new_col) # Allows this col to be used in next cols
+            self$data[[target]][[names(new_col)]] <- new_col[[1]]
+          }
+        }
+      } else { # not a current dataset
+        new_dataset <- list()
+        for (index in seq_along(unevaluated)) {
+          new_col <- lazyeval::lazy_eval(unevaluated[index], data = data_used)
+          data_used <- c(data_used, new_col) # Allows this col to be used in next cols
+          new_dataset <- c(new_dataset, new_col)
+        }
+        if (any(names(unevaluated) == "")) { # unnammed inputs cant be put in tables
+          if (length(unevaluated) == 1) { # Add as a vector
+            message('Adding a new "', class(new_dataset[[1]]),'" vector of length ', length(new_dataset[[1]]), '.')
+            self$data[[target]] <- new_dataset[[1]]
+          } else {
+            stop(call. = FALSE,
+                 "Cannot add a new dataset with multiple values if any are unnamed.",
+                 " The following input indexes are unnamed:\n",
+                 limited_print(which(names(unevaluated) == ""), type = "silent", prefix = "  "))
+          }
+        } else { # Try to put in new table
+          part_lengths <- vapply(new_dataset, length, numeric(1))
+          if (length(unique(part_lengths[part_lengths != 1])) == 1) { # All inputs are same length or 1
+            new_dataset <- dplyr::as_tibble(new_dataset)
+            message('Adding a new ', nrow(new_dataset), ' x ', ncol(new_dataset),
+                    ' table called "', target, '"')
+            self$data[[target]] <- dplyr::as_tibble(new_dataset)
+          } else {
+            stop(call. = FALSE,
+                 "Cannot make a new table out of multiple values of unequal length.",
+                 " The inputs have the following lengths:\n",
+                 limited_print(part_lengths, type = "silent", prefix = "  "))
+
+          }
+        }
       }
+
       return(self)
     },
 
 
+    # -------------------------------------------------------------------------
+    # Replace columns of tables in obj$data
     transmute_obs = function(target, ...) {
       # Check that the target data exists
-      private$check_dataset_name(target)
+      private$is_datset(target)
 
       # Check that the target is a table
       if (! is.data.frame(self$data[[target]])) {
@@ -330,9 +415,11 @@ Taxmap <- R6::R6Class(
     },
 
 
+    # -------------------------------------------------------------------------
+    # Sort columns of tables in obj$data
     arrange_obs = function(target, ...) {
       # Check that the target data exists
-      private$check_dataset_name(target)
+      private$is_datset(target)
 
       # Sort observations
       data_used <- self$data_used(...)
@@ -358,11 +445,13 @@ Taxmap <- R6::R6Class(
     },
 
 
+    # -------------------------------------------------------------------------
+    # Randomly sample some number of observations from a table
     sample_n_obs = function(target, size, replace = FALSE, taxon_weight = NULL,
                             obs_weight = NULL, use_supertaxa = TRUE,
                             collapse_func = mean, ...) {
       # Check that the target data exists
-      private$check_dataset_name(target)
+      private$is_datset(target)
 
       # non-standard argument evaluation
       data_used <- eval(substitute(self$data_used(taxon_weight, obs_weight)))
@@ -382,7 +471,7 @@ Taxmap <- R6::R6Class(
       if (is.null(taxon_weight)) {
         obs_taxon_weight <- rep(1, target_length)
       } else {
-        obs_index <- match(private$get_data_taxon_ids(target),
+        obs_index <- match(self$get_data_taxon_ids(target, require = TRUE),
                            self$taxon_ids())
         my_supertaxa <- self$supertaxa(recursive = use_supertaxa,
                                        simplify = FALSE, include_input = TRUE,
@@ -415,6 +504,8 @@ Taxmap <- R6::R6Class(
       self$filter_obs(target, sampled_rows, ...)
     },
 
+    # -------------------------------------------------------------------------
+    # Randomly sample some proportion of observations from a table
     sample_frac_obs = function(target, size, replace = FALSE,
                                taxon_weight = NULL, obs_weight = NULL,
                                use_supertaxa = TRUE,
@@ -427,15 +518,17 @@ Taxmap <- R6::R6Class(
       }
 
 
-      self$sample_n_obs(target = target,
-                        size = size * target_length,
-                        replace = replace,
-                        taxon_weight = taxon_weight, obs_weight = obs_weight,
-                        use_supertaxa = use_supertaxa,
-                        collapse_func = collapse_func, ...)
+      eval(substitute(self$sample_n_obs(target = target,
+                                        size = size * target_length,
+                                        replace = replace,
+                                        taxon_weight = taxon_weight, obs_weight = obs_weight,
+                                        use_supertaxa = use_supertaxa,
+                                        collapse_func = collapse_func, ...)))
     },
 
 
+    # -------------------------------------------------------------------------
+    # Count observations for each taxon in a data set
     n_obs = function(target = NULL) {
       if (is.null(target)) {
         if (length(self$data) > 0) {
@@ -448,6 +541,9 @@ Taxmap <- R6::R6Class(
              length, numeric(1))
     },
 
+    # -------------------------------------------------------------------------
+    # Count observations for each taxon in a data set, including observations
+    # for the specific taxon but NOT the observations of its subtaxa.
     n_obs_1 = function(target = NULL) {
       if (is.null(target)) {
         if (length(self$data) > 0) {
@@ -458,6 +554,72 @@ Taxmap <- R6::R6Class(
       }
       vapply(self$obs(target, recursive = FALSE, simplify = FALSE),
              length, numeric(1))
+    },
+
+    # Find taxon ids for datasets by dataset name
+    #
+    # require: if TRUE, require that taxon ids be present, or make an error
+    get_data_taxon_ids = function(dataset_name, require = FALSE, warn = FALSE) {
+      stop_or_warn <- function(text) {
+        if (require) {
+          stop(call. = FALSE, text)
+        }
+        if (warn) {
+          warning(call. = FALSE, text)
+        }
+      }
+
+      # Get the dataset
+      if (length(dataset_name) == 1 && # data is name/index of dataset in object
+          (dataset_name %in% names(self$data) || is.numeric(dataset_name))) {
+        dataset <- self$data[[dataset_name]]
+      } else { # it is an external data set, not in the object
+        dataset <- dataset_name
+        dataset_name <- deparse(substitute(dataset_name))
+      }
+
+      # Extract taxon ids if they exist
+      if (is.data.frame(dataset)) {
+        if ("taxon_id" %in% colnames(dataset)) {
+          is_valid <- private$ids_are_valid(dataset$taxon_id)
+          if (all(is_valid)) {
+            return(dataset$taxon_id)
+          } else  {
+            stop_or_warn(paste0('There is a "taxon_id" column in the data set "',
+                                dataset_name, '", but the following invalid IDs were found:\n  ',
+                                limited_print(dataset$taxon_id[! is_valid], type = "silent")))
+            return(NULL)
+          }
+        } else {
+          stop_or_warn(paste0('There is no "taxon_id" column in the data set "',
+                              dataset_name, '", so there are no taxon IDs.'))
+          return(NULL)
+        }
+      } else if (class(dataset) == "list" || is.vector(dataset)) {
+        if (! is.null(names(dataset))) {
+          is_valid <- private$ids_are_valid(names(dataset))
+          if (all(is_valid)) {
+            return(names(dataset))
+          } else if (all(! is_valid)) {
+            stop_or_warn(paste0('The data set "', dataset_name,
+                                '" is a named list/vector, but not named by taxon ids.'))
+            return(NULL)
+          } else { # some are valid, but not all
+            stop_or_warn(paste0('Dataset "', dataset_name, '" appears to be named by taxon IDs, but contains ', sum(! is_valid), ' invalid IDs:\n  ',
+                                limited_print(names(dataset)[! is_valid], type = "silent")))
+            return(NULL)
+          }
+        } else {
+          stop_or_warn(paste0('The data set "', dataset_name,
+                              '" is an unnamed list/vector, ',
+                              'so there are no taxon ids.'))
+          return(NULL)
+        }
+      } else {
+        stop_or_warn(paste0('I dont know how to extract taxon ids from dataset "', dataset_name,
+                            '" of type "', class(dataset)[1], '".'))
+        return(NULL)
+      }
     }
 
   ),
@@ -472,6 +634,8 @@ Taxmap <- R6::R6Class(
       "n_supertaxa_1",
       "n_subtaxa",
       "n_subtaxa_1",
+      "n_leaves",
+      "n_leaves_1",
       "taxon_ranks",
       "is_root",
       "is_stem",
@@ -482,15 +646,20 @@ Taxmap <- R6::R6Class(
       "n_obs_1"
     ),
 
-    check_dataset_name = function(target) {
-      if (! target %in% names(self$data)) {
+    is_datset = function(target, require = TRUE) {
+      if (target %in% names(self$data)) {
+        return(TRUE)
+      } else if (require) {
         stop(paste0("The target `", target, "` is not the name of a data set.",
                     " Valid targets include: ",
                     paste0(names(self$data), collapse = ", ")))
+      } else {
+        return(FALSE)
       }
     },
 
     # Remove observations from a particular dataset or just remove the taxon ids
+    # NOTE: indexes = what is NOT removed
     remove_obs = function(dataset, indexes, unname_only = FALSE) {
       if (unname_only) {
         if (is.data.frame(self$data[[dataset]])) {
@@ -508,43 +677,10 @@ Taxmap <- R6::R6Class(
       }
     },
 
-    # Find taxon ids for datasets by dataset name
-    #
-    # require: if TRUE, require that taxon ids be present, or make an error
-    get_data_taxon_ids = function(dataset_name, require = FALSE) {
-      # Get the dataset
-      if (length(dataset_name) == 1 && # data is name/index of dataset in object
-          (dataset_name %in% names(self$data) || is.numeric(dataset_name))) {
-        dataset <- self$data[[dataset_name]]
-      } else { # it is an external data set, not in the object
-        dataset <- dataset_name
-        dataset_name <- deparse(substitute(dataset_name))
-      }
-
-      # Extract taxon ids if they exist
-      if (is.data.frame(dataset)) {
-        if ("taxon_id" %in% colnames(dataset)) {
-          return(dataset$taxon_id)
-        } else if (require) {
-          stop(paste0('There is no "taxon_id" column in the data set "',
-                      dataset_name, '", so taxon ids cannot be extracted.'))
-        }
-      } else if (is.list(dataset) || is.vector(dataset)) {
-        if (! is.null(names(dataset))) {
-          return(names(dataset))
-        } else if (require) {
-          stop(paste0('The data set "', dataset_name,
-                      '" is a list/vector, but not named, ',
-                      'so taxon ids cannot be extracted.'))
-        }
-      } else if (require) {
-        stop(paste0('I dont know how to extract taxon ids from "', dataset_name,
-                    '" of type "', class(dataset)[1], '".'))
-      }
-
-      # Return NULL if taxon ids cannot be found
-      return(NULL)
+    # Checks if a character vector contains only taxon IDs.
+    # Returns logical vector same length as input
+    ids_are_valid = function(ids_to_check) {
+      ids_to_check %in% c(self$taxon_ids(), NA_character_)
     }
-
   )
 )
